"""Exceptions used to describe errors when handling
Azion API requests/responses.

Documentation of possible errors to be returned from the API can
be found here: https://api.azion.com/
"""


class AzionException(Exception):
    """Base class exception. All exceptions related to
    requests/responses errors are inherited from it.
    """
    pass


class AzionError(AzionException):
    """This class represents all errors related to response.

    Whenever Azion's API returns a error code, we catch them and
    provide a better and uniform way to handle/inspect them.

    .. attribute:: response

        HTTP response that originated the error.

    .. attribute:: status_code

        HTTP status code raised by the error.

    .. attribute:: errors

        List of errors generated from the response.
    """

    def __init__(self, response):
        self.response = response
        self.status_code = response.status_code
        self.errors = response.json()

    def __repr__(self):
        return '<{} [{}]>'.format(self.__class__.__name__,
                                  self.status_code)


class BadRequest(AzionError):
    """Indicate that the server could not understand the request due
    to invalid syntax.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/400
    """
    pass


class Unauthorized(AzionError):
    """Indicate that the request has not been applied because it lacks
    valid authentication credentials for the target resource.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/401
    """
    pass


class Forbidden(AzionError):
    """Indicate that the server understood the request but refuses to
    authorize it.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/403
    """
    pass

class NotFound(AzionError):
    """Indicate that the server can't find the requested resource.
    Links which lead to a 404 page are often called broken or dead links.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/404
    """
    pass


class MethodNotAllowed(AzionError):
    """Indicate that the request method is known by the server
    but has been disabled and cannot be used.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/405
    """
    pass


class NotAcceptable(AzionError):
    """Indicate that a response matching the list of acceptable values
    defined in Accept-Charset and Accept-Language cannot be served.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/406
    """
    pass


class Conflict(AzionError):
    """Indicate a request conflict with current state of the server.
    """
    pass


class TooManyRequests(AzionError):
    """Indicate he user has sent too many requests
    in a given amount of time ("rate limiting").

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/429
    """
    pass


class InternalServerError(AzionError):
    """Indicate that the server encountered an unexpected condition
    that prevented it from fulfilling the request.

    More info here: https://developer.mozilla.org/en-US/docs/Web/HTTP/Status/500
    """
    pass


error_handlers = {
    400: BadRequest,
    401: Unauthorized,
    403: Forbidden,
    404: NotFound,
    405: MethodNotAllowed,
    406: NotAcceptable,
    409: Conflict,
    429: TooManyRequests,
    500: InternalServerError
}


def handle_error(response):
    """Handle the request that failed to retrieve an appropriate
    response.

    :param object response:
        requests Response object.
    """
    handler = error_handlers.get(response.status_code)
    return handler(response)



VERSION = '3'
CLIENT_VERSION = '0.0.1.a0'
PUBLIC_ENDPOINT = 'https://api.azionapi.net'
USER_AGENT = "azion-api-client/%s" %  CLIENT_VERSION

DEFAULT_PAGE_SIZE = 100
DEFAULT_PARAMS = {'page_size': DEFAULT_PAGE_SIZE}



import pendulum

import azion.exceptions as exceptions


def instance_from_data(model, data):
    if not data:
        return None
    return model(data)


def many_of(model, data):
    if not data:
        return []
    return [instance_from_data(model, resource) for
            resource in data]


def decode_json(response, excepted_status_code):
    """Decode a JSON response.
    """

    # Bad request is interpreted as a falsey value.
    # So we compare it with `None`.
    if response is None:
        return None

    status_code = response.status_code
    if status_code != excepted_status_code:
        if status_code >= 400:
            raise exceptions.handle_error(response)

    return response.json()


def as_boolean(response, expected_status_code):
    if response:
        if response.status_code == expected_status_code:
            return True
        if response.status_code >= 400:
            raise exceptions.handle_error(response)
    return False


def filter_none(data):
    return {key: value for key, value in list(data.items()) if value is not None}


def to_date(date):
    """Convert a string to a datetime object.
    """
    return pendulum.parse(date)


class Token(object):
    """Model representing the authorized token retrieved
    from the API.
    """

    def __init__(self, data):
        self.value = data['token']
        self.created_at = to_date(data['created_at'])
        self.expires_at = to_date(data['expires_at'])

    def __repr__(self):
        return '<TokenAuth [{}]>'.format(self.value[:6])


class Configuration(object):
    """Model representing the configuration retrieved
    from the API.
    """

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        self.domain_name = data['domain_name']
        self.active = data['active']
        self.delivery_protocol = data['delivery_protocol']
        self.digital_certificate = data['digital_certificate']
        self.cname = data['cname']
        self.cname_access_only = data['cname_access_only']
        self.rawlogs = data['rawlogs']
        try:
            self.application_aceleration = data['application_aceleration']
        except (NameError, KeyError):
            self.application_aceleration = None
            pass

    def __repr__(self):
        return '<Configuration [{} ({})]>'.format(self.name,
                                                  self.domain_name)


class ErrorResponses(object):
    """Model representing the error responses configuration
    retrieved from the API.
    """

    def __init__(self, data):
        self.configuration_id = data['id']
        self.cache_error_400 = data['cache_error_400']
        self.cache_error_403 = data['cache_error_403']
        self.cache_error_404 = data['cache_error_404']
        self.cache_error_405 = data['cache_error_405']
        self.cache_error_414 = data['cache_error_414']
        self.cache_error_416 = data['cache_error_416']
        self.cache_error_501 = data['cache_error_501']

    def __repr__(self):
        return '<ErrorResponses [{}]>'.format(self.configuration_id)


class CacheSettings(object):
    """Model representing the cache settings configuration
    retrieved from the API.
    """

    def __init__(self, data):
        self.name = data['name']
        self.browser_cache_settings = data['browser_cache_settings']
        self.browser_cache_settings_maximum_ttl = data['browser_cache_settings_maximum_ttl']
        self.cdn_cache_settings = data['cdn_cache_settings']
        self.cdn_cache_settings_maximum_ttl = data['cdn_cache_settings_maximum_ttl']
        self.cache_by_query_string = data['cache_by_query_string']
        self.query_string_fields = data['query_string_fields']
        self.enable_query_string_sort = data['enable_query_string_sort']
        self.cache_by_cookies = data['cache_by_cookies']
        self.cookie_names = data['cookie_names']
        self.adaptive_delivery_action = data['adaptive_delivery_action']
        self.device_group = data['device_group']
        try:
            self.id = data['id']
        except (NameError, KeyError):
            pass
        try:
            self.enable_caching_for_post = data['enable_caching_for_post']
        except (NameError, KeyError):
            pass

    def __repr__(self):
        return '<CacheSettings [{} ({})]>'.format(self.name,
                                                  self.id)


class Rule(object):
    """Model representing the rule from rules engine
    configuration retrieved from the API.
    """

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        self.phase = data['phase']
        self.criteria = data['criteria']
        self.behaviors = data['behaviors']
        self.order = data['order']

    def __repr__(self):
        return '<Rule [{} ({})]>'.format(self.name,
                                         self.id)

class Domain(object):
    """Model representing the domain
    retrieved from the API.
    """

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        # TODO: implement the rest

    def __repr__(self):
        return '<Domain [{} ({})]>'.format(self.name,
                                         self.id)

class EdgeApplication(object):
    """Model representing the edge application
    retrieved from the API.
    """

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        self.active = data['active']
        # TODO: implement the rest
        # self.origins = []

    def __repr__(self):
        return '<EdgeApplication [{} ({})]>'.format(self.name,
                                         self.id)

class EdgeFunction(object):
    """Model representing the edge function
    retrieved from the API.
    """

    def __init__(self, data):
        self.id = data['id']
        self.name = data['name']
        self.code = data['code']
        self.active = data['active']
        self.language = data['language']
        self.modified = data['modified']
        self.json_args = data['json_args']
        self.last_editor = data['last_editor']
        self.initiator_type = data['initiator_type']
        self.reference_count = data['reference_count']

    def __repr__(self):
        return '<EdgeFunction [{} ({})]>'.format(self.name,
                                         self.id)



import azion.api
import click
import os
import sys
from pprint import pprint

AZION_USERNAME = os.getenv('AZION_USERNAME')
AZION_PASSWORD = os.getenv('AZION_PASSWORD')
AZION_API_TOKEN = os.getenv('AZION_API_TOKEN')

if AZION_API_TOKEN:
    client = azion.api.Client(token=AZION_API_TOKEN)
elif AZION_USERNAME and AZION_PASSWORD:
    client = azion.api.Client(
        username=AZION_USERNAME,
        password=AZION_PASSWORD
    )
else:
    sys.tracebacklimit = 0
    raise ValueError("AZION_API_TOKEN or (AZION_USERNAME and AZION_PASSWORD) must be provided")

@click.group()
def domain_group():
    pass

@domain_group.command("domain-list")
@click.option('--id', type=int)
def domain_list(id):
    if id is not None:
        domain = client.get_domain(id)
        print(domain)
    else:
        for domain in client.domains():
            print(domain)


@click.group()
def edge_function_group():
    pass

@edge_function_group.command("edge-function-list")
@click.option('--id', type=int)
def edge_function_list(id):
    if id is not None:
        edge_function = client.get_edge_function(id)
        print(edge_function)
    else:
        for edge_function in client.edge_functions():
            print(edge_function)


@edge_function_group.command("edge-function-info")
@click.option('--id', type=int, required=True)
def edge_function_info(id):
    edge_function = client.get_edge_function(id)
    pprint(vars(edge_function))


cli = click.CommandCollection(sources=[domain_group, edge_function_group])

if __name__ == '__main__':
    cli()




#!/usr/bin/env python
# -*- coding: utf-8 -*-

import io
from setuptools import find_packages, setup
from os import path


setup(
    name='azion-api-client',
    version='0.0.1.a0',
    description="A Python API client for Azion management and automation",
    long_description_content_type='text/markdown',
    url='https://github.com/diegosouza/azion-api-client',
    include_package_data=True,
    license='GPL3',
    author='Diego de Souza Mendes',
    author_email='diegosouza.br@gmail.com',
    python_requires='>=3.8',
    entry_points={"console_scripts": ["azion = azion.cli:cli"]},
    install_requires=[
        'pendulum',
        'requests',
        'click',
    ]
)
